@@ -24,7 +24,7 @@ class AnnotationClass(_instance_base.SimpleValue, mixin.HasSlots):
   def __init__(self, name, ctx):
     super().__init__(name, ctx)
     mixin.HasSlots.init_mixin(self)
-    self.set_slot("__getitem__", self.getitem_slot)
+    self.set_native_slot("__getitem__", self.getitem_slot)
 
   def getitem_slot(self, node, slice_var):
     """Custom __getitem__ implementation."""
@@ -323,11 +323,13 @@ class AnnotationContainer(AnnotationClass):
     return self._call_helper(node, self.base_cls, func, args)
 
 
-class TypeParameter(_base.BaseValue):
+class _TypeVariable(_base.BaseValue):
   """Parameter of a type."""
 
   formal = True
 
+  _INSTANCE_CLASS = None
+
   def __init__(self,
                name,
                ctx,
@@ -337,6 +339,8 @@ class TypeParameter(_base.BaseValue):
                contravariant=False,
                module=None):
     super().__init__(name, ctx)
+    # TODO(b/217789659): PEP-612 does not mention constraints, but ParamSpecs
+    # ignore all the extra parameters anyway..
     self.constraints = constraints
     self.bound = bound
     self.covariant = covariant
@@ -347,8 +351,8 @@ class TypeParameter(_base.BaseValue):
     return not self.constraints and not self.bound
 
   def copy(self):
-    return TypeParameter(self.name, self.ctx, self.constraints, self.bound,
-                         self.covariant, self.contravariant, self.module)
+    return self.__class__(self.name, self.ctx, self.constraints, self.bound,
+                          self.covariant, self.contravariant, self.module)
 
   def with_module(self, module):
     res = self.copy()
@@ -373,14 +377,15 @@ class TypeParameter(_base.BaseValue):
                  self.contravariant))
 
   def __repr__(self):
-    return ("TypeParameter({!r}, constraints={!r}, bound={!r}, module={!r})"
-            .format(self.name, self.constraints, self.bound, self.module))
+    return ("{!s}({!r}, constraints={!r}, bound={!r}, module={!r})"
+            .format(self.__class__.__name__, self.name, self.constraints,
+                    self.bound, self.module))
 
   def instantiate(self, node, container=None):
     var = self.ctx.program.NewVariable()
     if container and (not isinstance(container, _instance_base.SimpleValue) or
                       self.full_name in container.all_template_names):
-      instance = TypeParameterInstance(self, container, self.ctx)
+      instance = self._INSTANCE_CLASS(self, container, self.ctx)  # pylint: disable=not-callable
       return instance.to_variable(node)
     else:
       for c in self.constraints:
@@ -401,7 +406,7 @@ class TypeParameter(_base.BaseValue):
     return node, self.instantiate(node)
 
 
-class TypeParameterInstance(_base.BaseValue):
+class _TypeVariableInstance(_base.BaseValue):
   """An instance of a type parameter."""
 
   def __init__(self, param, instance, ctx):
@@ -417,9 +422,6 @@ class TypeParameterInstance(_base.BaseValue):
     else:
       return node, self.ctx.convert.empty.to_variable(self.ctx.root_node)
 
-  def __repr__(self):
-    return f"TypeParameterInstance({self.name!r})"
-
   def __eq__(self, other):
     if isinstance(other, type(self)):
       return self.param == other.param and self.instance == other.instance
@@ -428,6 +430,57 @@ class TypeParameterInstance(_base.BaseValue):
   def __hash__(self):
     return hash((self.param, self.instance))
 
+  def __repr__(self):
+    return f"{self.__class__.__name__}({self.name!r})"
+
+
+class TypeParameterInstance(_TypeVariableInstance):
+  """An instance of a TypeVar type parameter."""
+
+
+class ParamSpecInstance(_TypeVariableInstance):
+  """An instance of a ParamSpec type parameter."""
+
+
+class TypeParameter(_TypeVariable):
+  """Parameter of a type (typing.TypeVar)."""
+
+  _INSTANCE_CLASS = TypeParameterInstance
+
+
+class ParamSpec(_TypeVariable):
+  """Parameter of a callable type (typing.ParamSpec)."""
+
+  _INSTANCE_CLASS = ParamSpecInstance
+
+
+class Concatenate(_base.BaseValue):
+  """Concatenation of args and ParamSpec."""
+
+  def __init__(self, params, ctx):
+    super().__init__("Concatenate", ctx)
+    self.args = params[:-1]
+    self.paramspec = params[-1]
+
+  @property
+  def full_name(self):
+    return self.paramspec.full_name
+
+  def instantiate(self, node, container=None):
+    return self.to_variable(node)
+
+  @property
+  def num_args(self):
+    return len(self.args)
+
+  def get_args(self):
+    # Satisfies the same interface as abstract.CallableClass
+    return self.args
+
+  def __repr__(self):
+    args = ", ".join(list(map(repr, self.args)) + [self.paramspec.name])
+    return f"Concatenate[{args}]"
+
 
 class Union(_base.BaseValue, mixin.NestedAnnotation, mixin.HasSlots):
   """A list of types.
@@ -447,7 +500,7 @@ class Union(_base.BaseValue, mixin.NestedAnnotation, mixin.HasSlots):
     self._instance_cache = {}
     mixin.NestedAnnotation.init_mixin(self)
     mixin.HasSlots.init_mixin(self)
-    self.set_slot("__getitem__", self.getitem_slot)
+    self.set_native_slot("__getitem__", self.getitem_slot)
 
   def __repr__(self):
     if self._printing:  # recursion detected
