@@ -88,7 +88,7 @@ class Options:
       Postprocessor(names, opt_map, options, self).process()
     except PostprocessingError as e:
       if command_line:
-        argument_parser.error(utils.message(e))
+        argument_parser.error(str(e))
       else:
         raise
 
@@ -236,13 +236,11 @@ _OPT_IN_FEATURES = [
 FEATURE_FLAGS = [
     _flag("--always-use-return-annotations", False,
           "Always use function return type annotations."),
-    _flag("--mapping-is-not-sequence", True,
-          "Do not treat Mapping as satisfying the Sequence protocol."),
-    _flag("--overriding-default-value-checks", False,
+    _flag("--overriding-default-value-checks", True,
           "Enable default value checks for overriding methods."),
     _flag("--overriding-parameter-count-checks", False,
           "Enable parameter count checks for overriding methods."),
-    _flag("--overriding-parameter-name-checks", False,
+    _flag("--overriding-parameter-name-checks", True,
           "Enable parameter name checks for overriding methods."),
     _flag("--overriding-return-type-checks", False,
           "Enable return type checks for overriding methods."),
