@@ -1,5 +1,6 @@
 """Code for translating between type systems."""
 
+import contextlib
 import logging
 import types
 from typing import Any, Dict
@@ -67,6 +68,7 @@ class Converter(utils.ContextWeakrefMixin):
     ctx.convert = self  # to make constant_to_value calls below work
 
     self._convert_cache: Dict[Any, Any] = {}
+    self._skip_cache = False
     self._resolved_late_types = {}  # performance cache
 
     # Initialize primitive_classes to empty to allow constant_to_value to run.
@@ -141,6 +143,15 @@ class Converter(utils.ContextWeakrefMixin):
         None: self.primitive_class_instances[bool],
     }
 
+  @contextlib.contextmanager
+  def skip_cache(self):
+    old = self._skip_cache
+    self._skip_cache = True
+    try:
+      yield
+    finally:
+      self._skip_cache = old
+
   def constant_name(self, constant_type):
     if constant_type is None:
       return "constant"
@@ -380,6 +391,11 @@ class Converter(utils.ContextWeakrefMixin):
     # merged class type.
     return self.merge_values(sorted(classes, key=lambda cls: cls.full_name))
 
+  def convert_pytd_function(self, pytd_func, factory=abstract.PyTDFunction):
+    sigs = [abstract.PyTDSignature(pytd_func.name, sig, self.ctx)
+            for sig in pytd_func.signatures]
+    return factory(pytd_func.name, sigs, pytd_func.kind, self.ctx)
+
   def constant_to_var(self, pyval, subst=None, node=None, source_sets=None,
                       discard_concrete_values=False):
     """Convert a constant to a Variable.
@@ -490,7 +506,7 @@ class Converter(utils.ContextWeakrefMixin):
     else:
       type_key = type(pyval)
     key = ("constant", pyval, type_key)
-    if key in self._convert_cache:
+    if not self._skip_cache and key in self._convert_cache:
       if self._convert_cache[key] is None:
         self._convert_cache[key] = self.unsolvable
         # This error is triggered by, e.g., classes inheriting from each other.
@@ -610,7 +626,7 @@ class Converter(utils.ContextWeakrefMixin):
   def _special_constant_to_value(self, name):
     """Special-case construction of some pytd values."""
     if name == "builtins.super":
-      return self.ctx.special_builtins["super"]
+      return self.super_type
     elif name == "builtins.object":
       return self.object_type
     elif name == "types.ModuleType":
@@ -732,17 +748,7 @@ class Converter(utils.ContextWeakrefMixin):
           cls.call_metaclass_init(get_node())
         return cls
     elif isinstance(pyval, pytd.Function):
-      signatures = [
-          abstract.PyTDSignature(pyval.name, sig, self.ctx)
-          for sig in pyval.signatures
-      ]
-      type_new = self.ctx.loader.lookup_builtin("builtins.type").Lookup(
-          "__new__")
-      if pyval is type_new:
-        f_cls = special_builtins.TypeNew
-      else:
-        f_cls = abstract.PyTDFunction
-      f = f_cls(pyval.name, signatures, pyval.kind, self.ctx)
+      f = self.convert_pytd_function(pyval)
       f.is_abstract = pyval.is_abstract
       return f
     elif isinstance(pyval, pytd.ClassType):
@@ -778,19 +784,30 @@ class Converter(utils.ContextWeakrefMixin):
         return abstract.Union(options, self.ctx)
       else:
         return options[0]
-    elif isinstance(pyval, pytd.TypeParameter):
+    elif isinstance(pyval, (pytd.TypeParameter, pytd.ParamSpec)):
       constraints = tuple(
           self.constant_to_value(c, {}, self.ctx.root_node)
           for c in pyval.constraints)
       bound = (
           pyval.bound and
           self.constant_to_value(pyval.bound, {}, self.ctx.root_node))
-      return abstract.TypeParameter(
+      if isinstance(pyval, pytd.ParamSpec):
+        cls = abstract.ParamSpec
+      else:
+        cls = abstract.TypeParameter
+      return cls(
           pyval.name,
           self.ctx,
           constraints=constraints,
           bound=bound,
           module=pyval.scope)
+    elif isinstance(pyval, (pytd.ParamSpecArgs, pytd.ParamSpecKwargs)):
+      # TODO(b/217789659): Support these.
+      return self.unsolvable
+    elif isinstance(pyval, pytd.Concatenate):
+      params = [self.constant_to_value(param, subst, self.ctx.root_node)
+                for param in pyval.parameters]
+      return abstract.Concatenate(params, self.ctx)
     elif isinstance(pyval, abstract_utils.AsInstance):
       cls = pyval.cls
       if isinstance(cls, pytd.LateType):
