@@ -9,7 +9,6 @@ from typing import Dict, Iterable, List, Optional
 
 from pytype import file_utils
 from pytype import module_utils
-from pytype import utils
 from pytype.imports import base as imports_base
 from pytype.imports import builtin_stubs
 from pytype.imports import module_loader
@@ -107,7 +106,7 @@ class BadDependencyError(Exception):
     super().__init__(module_error + referenced)
 
   def __str__(self):
-    return utils.message(self)
+    return str(self.args[0])
 
 
 class _ModuleMap:
@@ -198,12 +197,11 @@ class _ModuleMap:
       assert m.module_name in self._modules
       for dependency in deps:
         module_prefix = dependency
-        while module_prefix == dependency or "." in module_prefix:
-          if module_prefix in self._modules:
-            break
-          module_prefix, _, _ = module_prefix.rpartition(".")
-        else:
-          raise KeyError(f"Module not found: {dependency}")
+        while module_prefix not in self._modules:
+          if "." in module_prefix:
+            module_prefix, _, _ = module_prefix.rpartition(".")
+          else:
+            raise KeyError(f"Module not found: {dependency}")
         todo.append(self._modules[module_prefix])
       newly_loaded_asts.append(loaded_ast)
       m.ast = loaded_ast.ast
@@ -252,8 +250,6 @@ class _Resolver:
         self.builtins_ast, full_names=False,
         allow_singletons=self.allow_singletons)
     mod_ast = self._lookup(bltn_lookup, mod_ast, lookup_ast)
-    mod_ast = mod_ast.Visit(
-        visitors.ExpandCompatibleBuiltins(self.builtins_ast))
     return mod_ast
 
   def resolve_external_types(self, mod_ast, module_map, aliases, *, mod_name):
@@ -262,7 +258,7 @@ class _Resolver:
       mod_ast = mod_ast.Visit(visitors.LookupExternalTypes(
           module_map, self_name=name, module_alias_map=aliases))
     except KeyError as e:
-      raise BadDependencyError(utils.message(e), name) from e
+      raise BadDependencyError(str(e), name) from e
     return mod_ast
 
   def resolve_module_alias(self, name, *, lookup_ast=None,
@@ -287,7 +283,7 @@ class _Resolver:
       mod_ast.Visit(visitors.VerifyLookup(ignore_late_types=True))
     except ValueError as e:
       name = mod_name or mod_ast.name
-      raise BadDependencyError(utils.message(e), name) from e
+      raise BadDependencyError(str(e), name) from e
     mod_ast.Visit(visitors.VerifyContainers())
     mod_ast.Visit(visitors.VerifyLiterals())
 
@@ -298,7 +294,7 @@ class _Resolver:
     mod_ast.Visit(deps)
     if isinstance(mod_ast, (pytd.TypeDeclUnit, pytd.Class)):
       return {k: v for k, v in deps.dependencies.items()
-              if not isinstance(mod_ast.Get(k), pytd.Class)}
+              if not isinstance(mod_ast.Get(k), (pytd.Class, pytd.ParamSpec))}
     else:
       return deps.dependencies
 
@@ -592,6 +588,10 @@ class Loader:
         # transitive imports, but lookups are expensive.
         dependencies = self._resolver.collect_dependencies(mod_ast)
         for k in dependencies:
+          while k not in self._modules:
+            if "." not in k:
+              break
+            k, _ = k.rsplit(".", 1)
           if k not in self._modules:
             assert mod_ast
             raise (
@@ -739,7 +739,7 @@ class PickledPyiLoader(Loader):
       ast = serialize_ast.ProcessAst(loaded_ast, self._modules.get_module_map())
     except serialize_ast.UnrestorableDependencyError as e:
       del self._modules[module_name]
-      raise BadDependencyError(utils.message(e), module_name) from e
+      raise BadDependencyError(str(e), module_name) from e
     # Mark all the module's late dependencies as explicitly imported.
     for d, _ in loaded_ast.late_dependencies:
       if d != loaded_ast.ast.name:
