@@ -571,6 +571,11 @@ class AbstractMatcher(utils.ContextWeakrefMixin):
       subst = subst.copy()
       subst[other_type.full_name] = new_var
       return subst
+    elif isinstance(other_type, abstract.ParamSpec):
+      # TODO(b/217789659): This is a very basic attempt at handling this case,
+      # and likely needs a lot more work
+      new_subst = {other_type.full_name: left.instantiate(self._node)}
+      return self._merge_substs(subst, [new_subst])
     elif (isinstance(other_type, typing_overlay.NoReturn) or
           isinstance(left, typing_overlay.NoReturn)):
       # `NoReturn` can only matches itself, `Any`, or `abstract.TypeParameter`.
@@ -630,6 +635,12 @@ class AbstractMatcher(utils.ContextWeakrefMixin):
       return subst
     elif isinstance(other_type, abstract.Empty):
       return self._match_type_against_type(left, other_type, subst, view)
+    elif isinstance(left, abstract.ParamSpecInstance):
+      new_subst = {left.full_name: other_type.instantiate(self._node)}
+      return self._merge_substs(subst, [new_subst])
+    elif isinstance(left, abstract.Concatenate):
+      new_subst = {left.full_name: other_type.instantiate(self._node)}
+      return self._merge_substs(subst, [new_subst])
     else:
       log.error("Invalid type: %s", type(other_type))
       return None
@@ -806,6 +817,9 @@ class AbstractMatcher(utils.ContextWeakrefMixin):
         if param.bindings and not any(v is left for v in param.data):
           return self._match_all_bindings(param, other_type, subst, view)
       return self._instantiate_and_match(left.param, other_type, subst, view)
+    elif isinstance(left, abstract.ParamSpecInstance):
+      new_subst = {left.name: other_type.instantiate(self._node)}
+      return self._merge_substs(subst, [new_subst])
     else:
       raise NotImplementedError("Matching not implemented for %s against %s" %
                                 (type(left), type(other_type)))
@@ -887,6 +901,22 @@ class AbstractMatcher(utils.ContextWeakrefMixin):
       return subst
     return match
 
+  def _match_signature_args_against_callable(
+      self, sig, other_type, subst, view, param_match):
+    """Helper function for _match_signature_against_callable."""
+    for name, expected_arg in zip(sig.param_names, other_type.get_args()):
+      actual_arg = sig.annotations.get(name, self.ctx.convert.unsolvable)
+      new_subst = param_match(actual_arg, expected_arg, subst)
+      if new_subst is None:
+        # Flip actual and expected, since argument types are contravariant.
+        subst = self._instantiate_and_match(
+            expected_arg, actual_arg, subst, view, container=other_type)
+        if subst is None:
+          return None
+      else:
+        subst = new_subst
+    return subst
+
   def _match_signature_against_callable(self, sig, other_type, subst, view):
     """Match a function.Signature against a parameterized callable."""
     # a special type param against type param matcher that takes priority over
@@ -905,24 +935,26 @@ class AbstractMatcher(utils.ContextWeakrefMixin):
     if not isinstance(other_type, abstract.CallableClass):
       # other_type does not specify argument types, so any arguments are fine.
       return subst
-    if sig.mandatory_param_count() > other_type.num_args:
-      return None
-    max_argcount = sig.maximum_param_count()
-    if max_argcount is not None and max_argcount < other_type.num_args:
-      return None
-    for name, expected_arg in zip(sig.param_names,
-                                  (other_type.formal_type_parameters[i]
-                                   for i in range(other_type.num_args))):
-      actual_arg = sig.annotations.get(name, self.ctx.convert.unsolvable)
-      new_subst = param_match(actual_arg, expected_arg, subst)
-      if new_subst is None:
-        # Flip actual and expected, since argument types are contravariant.
-        subst = self._instantiate_and_match(
-            expected_arg, actual_arg, subst, view, container=other_type)
+    # A ParamSpec can stand for any number of args, so handle it before we do
+    # argument count matching.
+    if other_type.has_paramspec():
+      expected_arg = other_type.formal_type_parameters[0]
+      if isinstance(expected_arg, abstract.Concatenate):
+        subst = self._match_signature_args_against_callable(
+            sig, expected_arg, subst, view, param_match)
         if subst is None:
           return None
-      else:
-        subst = new_subst
+      actual_arg = function.ParamSpecMatch(expected_arg, sig, self.ctx)
+      subst = self._instantiate_and_match(
+          expected_arg, actual_arg, subst, view, container=other_type)
+    else:
+      if sig.mandatory_param_count() > other_type.num_args:
+        return None
+      max_argcount = sig.maximum_param_count()
+      if max_argcount is not None and max_argcount < other_type.num_args:
+        return None
+      subst = self._match_signature_args_against_callable(
+          sig, other_type, subst, view, param_match)
     return subst
 
   def _merge_substs(self, subst, new_substs):
@@ -1219,6 +1251,24 @@ class AbstractMatcher(utils.ContextWeakrefMixin):
             return None
     return subst
 
+  def _match_callable_args_against_callable(self, left, other_type, subst,
+                                            view):
+    # TODO(mdemello): Unify with _match_signature_args_against_callable()
+    param_match = self._get_param_matcher(other_type)
+    for i in range(left.num_args):
+      left_arg = left.formal_type_parameters[i]
+      right_arg = other_type.formal_type_parameters[i]
+      new_subst = param_match(left_arg, right_arg, subst)
+      if new_subst is None:
+        # Flip actual and expected to enforce contravariance of argument types.
+        subst = self._instantiate_and_match(
+            right_arg, left_arg, subst, view, container=other_type)
+      else:
+        subst = new_subst
+      if subst is None:
+        return None
+    return subst
+
   def _match_callable_instance(self, left, instance, other_type, subst, view):
     """Used by _match_instance."""
     if (not isinstance(instance, abstract.SimpleValue) or
@@ -1233,21 +1283,24 @@ class AbstractMatcher(utils.ContextWeakrefMixin):
         not isinstance(other_type, abstract.CallableClass)):
       # One of the types doesn't specify arg types, so no need to check them.
       return subst
-    if left.num_args != other_type.num_args:
-      return None
-    param_match = self._get_param_matcher(other_type)
-    for i in range(left.num_args):
-      left_arg = left.formal_type_parameters[i]
-      right_arg = other_type.formal_type_parameters[i]
-      new_subst = param_match(left_arg, right_arg, subst)
-      if new_subst is None:
-        # Flip actual and expected to enforce contravariance of argument types.
-        subst = self._instantiate_and_match(
-            right_arg, left_arg, subst, view, container=other_type)
-      else:
-        subst = new_subst
-      if subst is None:
+    # A ParamSpec can stand for any number of args, so handle it before we do
+    # argument count matching.
+    if other_type.has_paramspec():
+      right_arg = other_type.formal_type_parameters[0]
+      if isinstance(right_arg, abstract.Concatenate):
+        subst = self._match_callable_args_against_callable(
+            left, right_arg, subst, view)
+        if subst is None:
+          return None
+      sig = function.Signature.from_callable(left)
+      left_arg = function.ParamSpecMatch(right_arg, sig, self.ctx)
+      subst = self._instantiate_and_match(
+          right_arg, left_arg, subst, view, container=other_type)
+    else:
+      if left.num_args != other_type.num_args:
         return None
+      subst = self._match_callable_args_against_callable(
+          left, other_type, subst, view)
     return subst
 
   def _match_dict_against_typed_dict(
@@ -1301,8 +1354,7 @@ class AbstractMatcher(utils.ContextWeakrefMixin):
       return subst
     elif left.cls.is_dynamic:  # pytype: disable=attribute-error
       return self._subst_with_type_parameters_from(subst, other_type)
-    elif (self.ctx.options.mapping_is_not_sequence and
-          other_type.full_name == "typing.Sequence" and
+    elif (other_type.full_name == "typing.Sequence" and
           any(cls.full_name == "typing.Mapping" for cls in left.cls.mro)):
       # A mapping should not be considered a sequence, even though pytype says
       # that dict[int | slice, str] satisfies the Sequence[str] protocol:
