@@ -313,8 +313,8 @@ class MatcherTest(MatcherTestBase):
     _, var = self.ctx.attribute_handler.get_attribute(self.ctx.root_node,
                                                       instance.cls, "f")
     unbound = var.data[0]
-    callable_no_self = self._convert_type("Callable[[int]]")
-    callable_self = self._convert_type("Callable[[Any, int]]")
+    callable_no_self = self._convert_type("Callable[[int], Any]")
+    callable_self = self._convert_type("Callable[[Any, int], Any]")
     self.assertMatch(bound, callable_no_self)
     self.assertNoMatch(unbound, callable_no_self)
     self.assertNoMatch(bound, callable_self)
@@ -441,7 +441,7 @@ class MatcherTest(MatcherTestBase):
     self.assertMatch(self.ctx.convert.no_return, self.ctx.convert.no_return)
 
   def test_empty_against_no_return(self):
-    self.assertNoMatch(self.ctx.convert.empty, self.ctx.convert.no_return)
+    self.assertMatch(self.ctx.convert.empty, self.ctx.convert.no_return)
 
   def test_no_return_against_class(self):
     right = self._convert_type("int")
