@@ -368,6 +368,32 @@ class TypeParameter(Node, Type):
       return AnythingType()
 
 
+@attrs.frozen(cache_hash=True)
+class ParamSpec(TypeParameter):
+  """ParamSpec is a specific case of TypeParameter."""
+
+  def Get(self, attr):
+    if attr == 'args':
+      return ParamSpecArgs(self.name)
+    elif attr == 'kwargs':
+      return ParamSpecKwargs(self.name)
+    else:
+      # TODO(b/217789659): This should be an error
+      return None
+
+
+@attrs.frozen(cache_hash=True)
+class ParamSpecArgs(Node):
+  """ParamSpec.args special form."""
+  name: str
+
+
+@attrs.frozen(cache_hash=True)
+class ParamSpecKwargs(Node):
+  """ParamSpec.kwargs special form."""
+  name: str
+
+
 @attrs.frozen(cache_hash=True)
 class TemplateItem(Node):
   """Represents template name for generic types.
@@ -596,6 +622,22 @@ class CallableType(GenericType):
   def ret(self):
     return self.parameters[-1]
 
+  def has_paramspec(self):
+    return self.args and isinstance(self.args[0], (ParamSpec, Concatenate))
+
+
+class Concatenate(GenericType):
+  """Concatenate params and ParamSpec."""
+  __slots__ = ()
+
+  @property
+  def args(self):
+    return self.parameters[:-1]
+
+  @property
+  def paramspec(self):
+    return self.parameters[-1]
+
 
 @attrs.frozen(cache_hash=True)
 class Literal(Node, Type):
@@ -627,7 +669,10 @@ def IsContainer(t: Class) -> bool:
   for p in t.bases:
     if isinstance(p, GenericType):
       base = p.base_type
-      if isinstance(base, ClassType) and IsContainer(base.cls):
+      # We consider a LateType a container, since we don't have enough
+      # information to determine whether it is one or not.
+      if (isinstance(base, ClassType) and IsContainer(base.cls) or
+          isinstance(base, LateType)):
         return True
   return False
 
@@ -662,6 +707,8 @@ def ToType(item, allow_constants=False, allow_functions=False,
     return item
   elif isinstance(item, Module):
     return item
+  elif isinstance(item, (ParamSpecArgs, ParamSpecKwargs)):
+    return item
   elif isinstance(item, Class):
     return ClassType(item.name, item)
   elif isinstance(item, Function) and allow_functions:
@@ -752,7 +799,7 @@ def LookupItemRecursive(module, name):
       if not found:
         raise KeyError(item.type.name)
       item = found
-    elif not isinstance(item, (TypeDeclUnit, Class)):
+    elif not isinstance(item, (TypeDeclUnit, Class, ParamSpec)):
       raise KeyError(name)
     lookup_name = partial_name + '.' + part
 
