@@ -92,21 +92,17 @@ class SerializableAst(SerializableTupleClass):
   Replace = SerializableTupleClass._replace  # pylint: disable=no-member,invalid-name
 
 
-def StoreAst(ast, filename=None, open_function=open, is_package=False,
-             src_path=None, metadata=None):
+def SerializeAst(ast, is_package=False, src_path=None, metadata=None):
   """Loads and stores an ast to disk.
 
   Args:
     ast: The pytd.TypeDeclUnit to save to disk.
-    filename: The filename for the pickled output. If this is None, this
-      function instead returns the pickled string.
-    open_function: A custom file opening function.
     is_package: Whether the module with the given ast is a package.
     src_path: Optionally, the filepath of the original source file.
     metadata: A list of arbitrary string-encoded metadata.
 
   Returns:
-    The pickled string, if no filename was given. (None otherwise.)
+    The SerializableAst derived from `ast`.
   """
   if ast.name.endswith(".__init__"):
     assert is_package
@@ -127,13 +123,11 @@ def StoreAst(ast, filename=None, open_function=open, is_package=False,
 
   metadata = metadata or []
 
-  return pytd_utils.SavePickle(
-      SerializableAst(
-          ast, sorted(dependencies.items()), sorted(late_dependencies.items()),
-          sorted(indexer.class_type_nodes), is_package=is_package,
-          src_path=src_path, metadata=metadata,
-      ),
-      filename, open_function=open_function)
+  return SerializableAst(
+      ast, sorted(dependencies.items()), sorted(late_dependencies.items()),
+      sorted(indexer.class_type_nodes), is_package=is_package,
+      src_path=src_path, metadata=metadata,
+  )
 
 
 def EnsureAstName(ast, module_name, fix=False):
@@ -231,13 +225,13 @@ def _LookupClassReferences(serializable_ast, module_map, self_name):
         break
     except KeyError as e:
       raise UnrestorableDependencyError(
-          f"Unresolved class: {utils.message(e)!r}.") from e
+          f"Unresolved class: {str(e)!r}.") from e
   if serializable_ast.class_type_nodes is None:
     try:
       raw_ast = raw_ast.Visit(class_lookup)
     except KeyError as e:
       raise UnrestorableDependencyError(
-          f"Unresolved class: {utils.message(e)!r}.") from e
+          f"Unresolved class: {str(e)!r}.") from e
   serializable_ast = serializable_ast.Replace(ast=raw_ast)
   return serializable_ast
 
@@ -290,7 +284,6 @@ def SourceToExportableAst(module_name, src, loader):
       src=src, name=module_name,
       options=parser.PyiOptions.from_toplevel_options(loader.options))
   ast = ast.Visit(visitors.LookupBuiltins(loader.builtins, full_names=False))
-  ast = ast.Visit(visitors.ExpandCompatibleBuiltins(loader.builtins))
   ast = ast.Visit(visitors.LookupLocalTypes())
   ast = ast.Visit(visitors.AdjustTypeParameters())
   ast = ast.Visit(visitors.NamedTypeToClassType())
