@@ -428,18 +428,21 @@ class Super(BuiltinClass):
     else:
       raise function.WrongArgCount(self._SIGNATURE, args, self.ctx)
     for cls in cls_var.bindings:
-      if not isinstance(cls.data, (abstract.Class,
-                                   abstract.AMBIGUOUS_OR_EMPTY)):
+      if isinstance(cls.data, (abstract.Class, abstract.AMBIGUOUS_OR_EMPTY)):
+        cls_data = cls.data
+      elif any(base.full_name == "builtins.type" for base in cls.data.cls.mro):
+        cls_data = self.ctx.convert.unsolvable
+      else:
         bad = abstract_utils.BadType(name="cls", typ=self.ctx.convert.type_type)
         raise function.WrongArgTypes(
             self._SIGNATURE, args, self.ctx, bad_param=bad)
       for obj in super_objects:
         if obj:
           result.AddBinding(
-              SuperInstance(cls.data, obj.data, self.ctx), [cls, obj], node)
+              SuperInstance(cls_data, obj.data, self.ctx), [cls, obj], node)
         else:
           result.AddBinding(
-              SuperInstance(cls.data, None, self.ctx), [cls], node)
+              SuperInstance(cls_data, None, self.ctx), [cls], node)
     return node, result
 
 
@@ -540,9 +543,6 @@ class PropertyTemplate(BuiltinClass):
 
   _KEYS = ["fget", "fset", "fdel", "doc"]
 
-  def __init__(self, ctx, name, module="builtins"):  # pylint: disable=useless-super-delegation
-    super().__init__(ctx, name, module)
-
   def signature(self):
     # Minimal signature, only used for constructing exceptions.
     return function.Signature.from_param_names(self.name, tuple(self._KEYS))
@@ -577,12 +577,12 @@ class PropertyInstance(abstract.Function, mixin.HasSlots):
     self.fdel = fdel
     self.doc = doc
     self.cls = cls
-    self.set_slot("__get__", self.fget_slot)
-    self.set_slot("__set__", self.fset_slot)
-    self.set_slot("__delete__", self.fdelete_slot)
-    self.set_slot("getter", self.getter_slot)
-    self.set_slot("setter", self.setter_slot)
-    self.set_slot("deleter", self.deleter_slot)
+    self.set_native_slot("__get__", self.fget_slot)
+    self.set_native_slot("__set__", self.fset_slot)
+    self.set_native_slot("__delete__", self.fdelete_slot)
+    self.set_native_slot("getter", self.getter_slot)
+    self.set_native_slot("setter", self.setter_slot)
+    self.set_native_slot("deleter", self.deleter_slot)
     self.is_abstract = any(_is_fn_abstract(x) for x in [fget, fset, fdel])
     self.is_method = True
     self.bound_class = abstract.BoundFunction
@@ -676,7 +676,7 @@ class StaticMethodInstance(abstract.Function, mixin.HasSlots):
     mixin.HasSlots.init_mixin(self)
     self.func = func
     self.cls = cls
-    self.set_slot("__get__", self.func_slot)
+    self.set_native_slot("__get__", self.func_slot)
     self.is_abstract = _is_fn_abstract(func)
     self.is_method = True
     self.bound_class = abstract.BoundFunction
@@ -715,7 +715,7 @@ class ClassMethodInstance(abstract.Function, mixin.HasSlots):
     mixin.HasSlots.init_mixin(self)
     self.cls = cls
     self.func = func
-    self.set_slot("__get__", self.func_slot)
+    self.set_native_slot("__get__", self.func_slot)
     self.is_abstract = _is_fn_abstract(func)
     self.is_method = True
     self.bound_class = ClassMethodCallable
@@ -760,3 +760,19 @@ class Dict(BuiltinClass):
       return node, d.to_variable(node)
     else:
       return super().call(node, funcb, args)
+
+
+class TypeTemplate(BuiltinClass, mixin.HasSlots):
+
+  def __init__(self, *args, **kwargs):
+    super().__init__(*args, **kwargs)
+    mixin.HasSlots.init_mixin(self)
+    slot = self.ctx.convert.convert_pytd_function(
+        self.pytd_cls.Lookup("__new__"), TypeNew)
+    self.set_slot("__new__", slot)
+
+
+class Type(TypeTemplate):
+
+  def __init__(self, ctx):
+    super().__init__(ctx, "type")
